@@ -1,8 +1,8 @@
-// A Bison parser, made by GNU Bison 3.7.4.
+// A Bison parser, made by GNU Bison 3.8.2.
 
 // Skeleton interface for Bison LALR(1) parsers in C++
 
-// Copyright (C) 2002-2015, 2018-2020 Free Software Foundation, Inc.
+// Copyright (C) 2002-2015, 2018-2021 Free Software Foundation, Inc.
 
 // This program is free software: you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -15,7 +15,7 @@
 // GNU General Public License for more details.
 
 // You should have received a copy of the GNU General Public License
-// along with this program.  If not, see <http://www.gnu.org/licenses/>.
+// along with this program.  If not, see <https://www.gnu.org/licenses/>.
 
 // As a special exception, you may create a larger work that contains
 // part or all of the Bison parser skeleton and distribute that work
@@ -127,17 +127,23 @@ using namespace isc::eval;
 
 /* Suppress unused-variable warnings by "using" E.  */
 #if ! defined lint || defined __GNUC__
-# define YYUSE(E) ((void) (E))
+# define YY_USE(E) ((void) (E))
 #else
-# define YYUSE(E) /* empty */
+# define YY_USE(E) /* empty */
 #endif
 
-#if defined __GNUC__ && ! defined __ICC && 407 <= __GNUC__ * 100 + __GNUC_MINOR__
 /* Suppress an incorrect diagnostic about yylval being uninitialized.  */
-# define YY_IGNORE_MAYBE_UNINITIALIZED_BEGIN                            \
+#if defined __GNUC__ && ! defined __ICC && 406 <= __GNUC__ * 100 + __GNUC_MINOR__
+# if __GNUC__ * 100 + __GNUC_MINOR__ < 407
+#  define YY_IGNORE_MAYBE_UNINITIALIZED_BEGIN                           \
+    _Pragma ("GCC diagnostic push")                                     \
+    _Pragma ("GCC diagnostic ignored \"-Wuninitialized\"")
+# else
+#  define YY_IGNORE_MAYBE_UNINITIALIZED_BEGIN                           \
     _Pragma ("GCC diagnostic push")                                     \
     _Pragma ("GCC diagnostic ignored \"-Wuninitialized\"")              \
     _Pragma ("GCC diagnostic ignored \"-Wmaybe-uninitialized\"")
+# endif
 # define YY_IGNORE_MAYBE_UNINITIALIZED_END      \
     _Pragma ("GCC diagnostic pop")
 #else
@@ -199,7 +205,7 @@ using namespace isc::eval;
 
 #line 14 "parser.yy"
 namespace isc { namespace eval {
-#line 203 "parser.h"
+#line 209 "parser.h"
 
 
 
@@ -208,27 +214,32 @@ namespace isc { namespace eval {
   class EvalParser
   {
   public:
-#ifndef EVALSTYPE
+#ifdef EVALSTYPE
+# ifdef __GNUC__
+#  pragma GCC message "bison: do not #define EVALSTYPE in C++, use %define api.value.type"
+# endif
+    typedef EVALSTYPE value_type;
+#else
   /// A buffer to store and retrieve objects.
   ///
   /// Sort of a variant, but does not keep track of the nature
   /// of the stored data, since that knowledge is available
   /// via the current parser state.
-  class semantic_type
+  class value_type
   {
   public:
     /// Type of *this.
-    typedef semantic_type self_type;
+    typedef value_type self_type;
 
     /// Empty construction.
-    semantic_type () YY_NOEXCEPT
-      : yybuffer_ ()
+    value_type () YY_NOEXCEPT
+      : yyraw_ ()
       , yytypeid_ (YY_NULLPTR)
     {}
 
     /// Construct and fill.
     template <typename T>
-    semantic_type (YY_RVREF (T) t)
+    value_type (YY_RVREF (T) t)
       : yytypeid_ (&typeid (T))
     {
       EVAL_ASSERT (sizeof (T) <= size);
@@ -237,13 +248,13 @@ namespace isc { namespace eval {
 
 #if 201103L <= YY_CPLUSPLUS
     /// Non copyable.
-    semantic_type (const self_type&) = delete;
+    value_type (const self_type&) = delete;
     /// Non copyable.
     self_type& operator= (const self_type&) = delete;
 #endif
 
     /// Destruction, allowed only if empty.
-    ~semantic_type () YY_NOEXCEPT
+    ~value_type () YY_NOEXCEPT
     {
       EVAL_ASSERT (!yytypeid_);
     }
@@ -387,7 +398,7 @@ namespace isc { namespace eval {
   private:
 #if YY_CPLUSPLUS < 201103L
     /// Non copyable.
-    semantic_type (const self_type&);
+    value_type (const self_type&);
     /// Non copyable.
     self_type& operator= (const self_type&);
 #endif
@@ -397,7 +408,7 @@ namespace isc { namespace eval {
     T*
     yyas_ () YY_NOEXCEPT
     {
-      void *yyp = yybuffer_.yyraw;
+      void *yyp = yyraw_;
       return static_cast<T*> (yyp);
      }
 
@@ -406,7 +417,7 @@ namespace isc { namespace eval {
     const T*
     yyas_ () const YY_NOEXCEPT
     {
-      const void *yyp = yybuffer_.yyraw;
+      const void *yyp = yyraw_;
       return static_cast<const T*> (yyp);
      }
 
@@ -454,18 +465,19 @@ namespace isc { namespace eval {
     union
     {
       /// Strongest alignment constraints.
-      long double yyalign_me;
+      long double yyalign_me_;
       /// A buffer large enough to store any of the semantic values.
-      char yyraw[size];
-    } yybuffer_;
+      char yyraw_[size];
+    };
 
     /// Whether the content is built: if defined, the name of the stored type.
     const std::type_info *yytypeid_;
   };
 
-#else
-    typedef EVALSTYPE semantic_type;
 #endif
+    /// Backward compatibility (Bison 3.8).
+    typedef value_type semantic_type;
+
     /// Symbol locations.
     typedef location location_type;
 
@@ -528,33 +540,42 @@ namespace isc { namespace eval {
     TOKEN_YIADDR = 287,            // "yiaddr"
     TOKEN_SIADDR = 288,            // "siaddr"
     TOKEN_SUBSTRING = 289,         // "substring"
-    TOKEN_ALL = 290,               // "all"
-    TOKEN_COMA = 291,              // ","
-    TOKEN_CONCAT = 292,            // "concat"
-    TOKEN_IFELSE = 293,            // "ifelse"
-    TOKEN_TOHEXSTRING = 294,       // "hexstring"
-    TOKEN_PKT6 = 295,              // "pkt6"
-    TOKEN_MSGTYPE = 296,           // "msgtype"
-    TOKEN_TRANSID = 297,           // "transid"
-    TOKEN_VENDOR_CLASS = 298,      // "vendor-class"
-    TOKEN_VENDOR = 299,            // "vendor"
-    TOKEN_ANY = 300,               // "*"
-    TOKEN_DATA = 301,              // "data"
-    TOKEN_ENTERPRISE = 302,        // "enterprise"
-    TOKEN_TOPLEVEL_BOOL = 303,     // "top-level bool"
-    TOKEN_TOPLEVEL_STRING = 304,   // "top-level string"
-    TOKEN_STRING = 305,            // "constant string"
-    TOKEN_INTEGER = 306,           // "integer"
-    TOKEN_HEXSTRING = 307,         // "constant hexstring"
-    TOKEN_OPTION_NAME = 308,       // "option name"
-    TOKEN_IP_ADDRESS = 309         // "ip address"
+    TOKEN_SPLIT = 290,             // "split"
+    TOKEN_ALL = 291,               // "all"
+    TOKEN_COMA = 292,              // ","
+    TOKEN_CONCAT = 293,            // "concat"
+    TOKEN_PLUS = 294,              // "+"
+    TOKEN_IFELSE = 295,            // "ifelse"
+    TOKEN_TOHEXSTRING = 296,       // "hexstring"
+    TOKEN_ADDRTOTEXT = 297,        // "addrtotext"
+    TOKEN_INT8TOTEXT = 298,        // "int8totext"
+    TOKEN_INT16TOTEXT = 299,       // "int16totext"
+    TOKEN_INT32TOTEXT = 300,       // "int32totext"
+    TOKEN_UINT8TOTEXT = 301,       // "uint8totext"
+    TOKEN_UINT16TOTEXT = 302,      // "uint16totext"
+    TOKEN_UINT32TOTEXT = 303,      // "uint32totext"
+    TOKEN_PKT6 = 304,              // "pkt6"
+    TOKEN_MSGTYPE = 305,           // "msgtype"
+    TOKEN_TRANSID = 306,           // "transid"
+    TOKEN_VENDOR_CLASS = 307,      // "vendor-class"
+    TOKEN_VENDOR = 308,            // "vendor"
+    TOKEN_ANY = 309,               // "*"
+    TOKEN_DATA = 310,              // "data"
+    TOKEN_ENTERPRISE = 311,        // "enterprise"
+    TOKEN_TOPLEVEL_BOOL = 312,     // "top-level bool"
+    TOKEN_TOPLEVEL_STRING = 313,   // "top-level string"
+    TOKEN_STRING = 314,            // "constant string"
+    TOKEN_INTEGER = 315,           // "integer"
+    TOKEN_HEXSTRING = 316,         // "constant hexstring"
+    TOKEN_OPTION_NAME = 317,       // "option name"
+    TOKEN_IP_ADDRESS = 318         // "ip address"
       };
       /// Backward compatibility alias (Bison 3.6).
       typedef token_kind_type yytokentype;
     };
 
     /// Token kind, as returned by yylex.
-    typedef token::yytokentype token_kind_type;
+    typedef token::token_kind_type token_kind_type;
 
     /// Backward compatibility alias (Bison 3.6).
     typedef token_kind_type token_type;
@@ -564,7 +585,7 @@ namespace isc { namespace eval {
     {
       enum symbol_kind_type
       {
-        YYNTOKENS = 55, ///< Number of tokens.
+        YYNTOKENS = 64, ///< Number of tokens.
         S_YYEMPTY = -2,
         S_YYEOF = 0,                             // "end of file"
         S_YYerror = 1,                           // error
@@ -601,43 +622,53 @@ namespace isc { namespace eval {
         S_YIADDR = 32,                           // "yiaddr"
         S_SIADDR = 33,                           // "siaddr"
         S_SUBSTRING = 34,                        // "substring"
-        S_ALL = 35,                              // "all"
-        S_COMA = 36,                             // ","
-        S_CONCAT = 37,                           // "concat"
-        S_IFELSE = 38,                           // "ifelse"
-        S_TOHEXSTRING = 39,                      // "hexstring"
-        S_PKT6 = 40,                             // "pkt6"
-        S_MSGTYPE = 41,                          // "msgtype"
-        S_TRANSID = 42,                          // "transid"
-        S_VENDOR_CLASS = 43,                     // "vendor-class"
-        S_VENDOR = 44,                           // "vendor"
-        S_ANY = 45,                              // "*"
-        S_DATA = 46,                             // "data"
-        S_ENTERPRISE = 47,                       // "enterprise"
-        S_TOPLEVEL_BOOL = 48,                    // "top-level bool"
-        S_TOPLEVEL_STRING = 49,                  // "top-level string"
-        S_STRING = 50,                           // "constant string"
-        S_INTEGER = 51,                          // "integer"
-        S_HEXSTRING = 52,                        // "constant hexstring"
-        S_OPTION_NAME = 53,                      // "option name"
-        S_IP_ADDRESS = 54,                       // "ip address"
-        S_YYACCEPT = 55,                         // $accept
-        S_start = 56,                            // start
-        S_expression = 57,                       // expression
-        S_bool_expr = 58,                        // bool_expr
-        S_string_expr = 59,                      // string_expr
-        S_integer_expr = 60,                     // integer_expr
-        S_option_code = 61,                      // option_code
-        S_sub_option_code = 62,                  // sub_option_code
-        S_option_repr_type = 63,                 // option_repr_type
-        S_nest_level = 64,                       // nest_level
-        S_pkt_metadata = 65,                     // pkt_metadata
-        S_enterprise_id = 66,                    // enterprise_id
-        S_pkt4_field = 67,                       // pkt4_field
-        S_pkt6_field = 68,                       // pkt6_field
-        S_relay6_field = 69,                     // relay6_field
-        S_start_expr = 70,                       // start_expr
-        S_length_expr = 71                       // length_expr
+        S_SPLIT = 35,                            // "split"
+        S_ALL = 36,                              // "all"
+        S_COMA = 37,                             // ","
+        S_CONCAT = 38,                           // "concat"
+        S_PLUS = 39,                             // "+"
+        S_IFELSE = 40,                           // "ifelse"
+        S_TOHEXSTRING = 41,                      // "hexstring"
+        S_ADDRTOTEXT = 42,                       // "addrtotext"
+        S_INT8TOTEXT = 43,                       // "int8totext"
+        S_INT16TOTEXT = 44,                      // "int16totext"
+        S_INT32TOTEXT = 45,                      // "int32totext"
+        S_UINT8TOTEXT = 46,                      // "uint8totext"
+        S_UINT16TOTEXT = 47,                     // "uint16totext"
+        S_UINT32TOTEXT = 48,                     // "uint32totext"
+        S_PKT6 = 49,                             // "pkt6"
+        S_MSGTYPE = 50,                          // "msgtype"
+        S_TRANSID = 51,                          // "transid"
+        S_VENDOR_CLASS = 52,                     // "vendor-class"
+        S_VENDOR = 53,                           // "vendor"
+        S_ANY = 54,                              // "*"
+        S_DATA = 55,                             // "data"
+        S_ENTERPRISE = 56,                       // "enterprise"
+        S_TOPLEVEL_BOOL = 57,                    // "top-level bool"
+        S_TOPLEVEL_STRING = 58,                  // "top-level string"
+        S_STRING = 59,                           // "constant string"
+        S_INTEGER = 60,                          // "integer"
+        S_HEXSTRING = 61,                        // "constant hexstring"
+        S_OPTION_NAME = 62,                      // "option name"
+        S_IP_ADDRESS = 63,                       // "ip address"
+        S_YYACCEPT = 64,                         // $accept
+        S_start = 65,                            // start
+        S_expression = 66,                       // expression
+        S_bool_expr = 67,                        // bool_expr
+        S_string_expr = 68,                      // string_expr
+        S_integer_expr = 69,                     // integer_expr
+        S_option_code = 70,                      // option_code
+        S_sub_option_code = 71,                  // sub_option_code
+        S_option_repr_type = 72,                 // option_repr_type
+        S_nest_level = 73,                       // nest_level
+        S_pkt_metadata = 74,                     // pkt_metadata
+        S_enterprise_id = 75,                    // enterprise_id
+        S_pkt4_field = 76,                       // pkt4_field
+        S_pkt6_field = 77,                       // pkt6_field
+        S_relay6_field = 78,                     // relay6_field
+        S_start_expr = 79,                       // start_expr
+        S_length_expr = 80,                      // length_expr
+        S_int_expr = 81                          // int_expr
       };
     };
 
@@ -660,7 +691,7 @@ namespace isc { namespace eval {
       typedef Base super_type;
 
       /// Default constructor.
-      basic_symbol ()
+      basic_symbol () YY_NOEXCEPT
         : value ()
         , location ()
       {}
@@ -871,8 +902,10 @@ namespace isc { namespace eval {
         clear ();
       }
 
+
+
       /// Destroy contents, and record that is empty.
-      void clear ()
+      void clear () YY_NOEXCEPT
       {
         // User destructor.
         symbol_kind_type yykind = this->kind ();
@@ -952,7 +985,7 @@ switch (yykind)
       void move (basic_symbol& s);
 
       /// The semantic value.
-      semantic_type value;
+      value_type value;
 
       /// The location.
       location_type location;
@@ -967,25 +1000,27 @@ switch (yykind)
     /// Type access provider for token (enum) based symbols.
     struct by_kind
     {
-      /// Default constructor.
-      by_kind ();
-
-#if 201103L <= YY_CPLUSPLUS
-      /// Move constructor.
-      by_kind (by_kind&& that);
-#endif
-
-      /// Copy constructor.
-      by_kind (const by_kind& that);
-
       /// The symbol kind as needed by the constructor.
       typedef token_kind_type kind_type;
 
+      /// Default constructor.
+      by_kind () YY_NOEXCEPT;
+
+#if 201103L <= YY_CPLUSPLUS
+      /// Move constructor.
+      by_kind (by_kind&& that) YY_NOEXCEPT;
+#endif
+
+      /// Copy constructor.
+      by_kind (const by_kind& that) YY_NOEXCEPT;
+
       /// Constructor from (external) token numbers.
-      by_kind (kind_type t);
+      by_kind (kind_type t) YY_NOEXCEPT;
+
+
 
       /// Record that this symbol is empty.
-      void clear ();
+      void clear () YY_NOEXCEPT;
 
       /// Steal the symbol kind from \a that.
       void move (by_kind& that);
@@ -1012,29 +1047,33 @@ switch (yykind)
       typedef basic_symbol<by_kind> super_type;
 
       /// Empty symbol.
-      symbol_type () {}
+      symbol_type () YY_NOEXCEPT {}
 
       /// Constructor for valueless symbols, and symbols from each type.
 #if 201103L <= YY_CPLUSPLUS
       symbol_type (int tok, location_type l)
-        : super_type(token_type (tok), std::move (l))
+        : super_type (token_kind_type (tok), std::move (l))
 #else
       symbol_type (int tok, const location_type& l)
-        : super_type(token_type (tok), l)
+        : super_type (token_kind_type (tok), l)
 #endif
       {
+#if !defined _MSC_VER || defined __clang__
         EVAL_ASSERT (tok == token::TOKEN_END
                    || (token::TOKEN_EVALerror <= tok && tok <= token::TOKEN_TOPLEVEL_STRING));
+#endif
       }
 #if 201103L <= YY_CPLUSPLUS
       symbol_type (int tok, std::string v, location_type l)
-        : super_type(token_type (tok), std::move (v), std::move (l))
+        : super_type (token_kind_type (tok), std::move (v), std::move (l))
 #else
       symbol_type (int tok, const std::string& v, const location_type& l)
-        : super_type(token_type (tok), v, l)
+        : super_type (token_kind_type (tok), v, l)
 #endif
       {
+#if !defined _MSC_VER || defined __clang__
         EVAL_ASSERT ((token::TOKEN_STRING <= tok && tok <= token::TOKEN_IP_ADDRESS));
+#endif
       }
     };
 
@@ -1083,7 +1122,7 @@ switch (yykind)
     /// YYSYMBOL.  No bounds checking.
     static std::string symbol_name (symbol_kind_type yysymbol);
 
-    // Implementation of make_symbol for each symbol type.
+    // Implementation of make_symbol for each token kind.
 #if 201103L <= YY_CPLUSPLUS
       static
       symbol_type
@@ -1609,6 +1648,21 @@ switch (yykind)
         return symbol_type (token::TOKEN_SUBSTRING, l);
       }
 #endif
+#if 201103L <= YY_CPLUSPLUS
+      static
+      symbol_type
+      make_SPLIT (location_type l)
+      {
+        return symbol_type (token::TOKEN_SPLIT, std::move (l));
+      }
+#else
+      static
+      symbol_type
+      make_SPLIT (const location_type& l)
+      {
+        return symbol_type (token::TOKEN_SPLIT, l);
+      }
+#endif
 #if 201103L <= YY_CPLUSPLUS
       static
       symbol_type
@@ -1654,6 +1708,21 @@ switch (yykind)
         return symbol_type (token::TOKEN_CONCAT, l);
       }
 #endif
+#if 201103L <= YY_CPLUSPLUS
+      static
+      symbol_type
+      make_PLUS (location_type l)
+      {
+        return symbol_type (token::TOKEN_PLUS, std::move (l));
+      }
+#else
+      static
+      symbol_type
+      make_PLUS (const location_type& l)
+      {
+        return symbol_type (token::TOKEN_PLUS, l);
+      }
+#endif
 #if 201103L <= YY_CPLUSPLUS
       static
       symbol_type
@@ -1684,6 +1753,111 @@ switch (yykind)
         return symbol_type (token::TOKEN_TOHEXSTRING, l);
       }
 #endif
+#if 201103L <= YY_CPLUSPLUS
+      static
+      symbol_type
+      make_ADDRTOTEXT (location_type l)
+      {
+        return symbol_type (token::TOKEN_ADDRTOTEXT, std::move (l));
+      }
+#else
+      static
+      symbol_type
+      make_ADDRTOTEXT (const location_type& l)
+      {
+        return symbol_type (token::TOKEN_ADDRTOTEXT, l);
+      }
+#endif
+#if 201103L <= YY_CPLUSPLUS
+      static
+      symbol_type
+      make_INT8TOTEXT (location_type l)
+      {
+        return symbol_type (token::TOKEN_INT8TOTEXT, std::move (l));
+      }
+#else
+      static
+      symbol_type
+      make_INT8TOTEXT (const location_type& l)
+      {
+        return symbol_type (token::TOKEN_INT8TOTEXT, l);
+      }
+#endif
+#if 201103L <= YY_CPLUSPLUS
+      static
+      symbol_type
+      make_INT16TOTEXT (location_type l)
+      {
+        return symbol_type (token::TOKEN_INT16TOTEXT, std::move (l));
+      }
+#else
+      static
+      symbol_type
+      make_INT16TOTEXT (const location_type& l)
+      {
+        return symbol_type (token::TOKEN_INT16TOTEXT, l);
+      }
+#endif
+#if 201103L <= YY_CPLUSPLUS
+      static
+      symbol_type
+      make_INT32TOTEXT (location_type l)
+      {
+        return symbol_type (token::TOKEN_INT32TOTEXT, std::move (l));
+      }
+#else
+      static
+      symbol_type
+      make_INT32TOTEXT (const location_type& l)
+      {
+        return symbol_type (token::TOKEN_INT32TOTEXT, l);
+      }
+#endif
+#if 201103L <= YY_CPLUSPLUS
+      static
+      symbol_type
+      make_UINT8TOTEXT (location_type l)
+      {
+        return symbol_type (token::TOKEN_UINT8TOTEXT, std::move (l));
+      }
+#else
+      static
+      symbol_type
+      make_UINT8TOTEXT (const location_type& l)
+      {
+        return symbol_type (token::TOKEN_UINT8TOTEXT, l);
+      }
+#endif
+#if 201103L <= YY_CPLUSPLUS
+      static
+      symbol_type
+      make_UINT16TOTEXT (location_type l)
+      {
+        return symbol_type (token::TOKEN_UINT16TOTEXT, std::move (l));
+      }
+#else
+      static
+      symbol_type
+      make_UINT16TOTEXT (const location_type& l)
+      {
+        return symbol_type (token::TOKEN_UINT16TOTEXT, l);
+      }
+#endif
+#if 201103L <= YY_CPLUSPLUS
+      static
+      symbol_type
+      make_UINT32TOTEXT (location_type l)
+      {
+        return symbol_type (token::TOKEN_UINT32TOTEXT, std::move (l));
+      }
+#else
+      static
+      symbol_type
+      make_UINT32TOTEXT (const location_type& l)
+      {
+        return symbol_type (token::TOKEN_UINT32TOTEXT, l);
+      }
+#endif
 #if 201103L <= YY_CPLUSPLUS
       static
       symbol_type
@@ -1915,9 +2089,9 @@ switch (yykind)
     {
     public:
       context (const EvalParser& yyparser, const symbol_type& yyla);
-      const symbol_type& lookahead () const { return yyla_; }
-      symbol_kind_type token () const { return yyla_.kind (); }
-      const location_type& location () const { return yyla_.location; }
+      const symbol_type& lookahead () const YY_NOEXCEPT { return yyla_; }
+      symbol_kind_type token () const YY_NOEXCEPT { return yyla_.kind (); }
+      const location_type& location () const YY_NOEXCEPT { return yyla_.location; }
 
       /// Put in YYARG at most YYARGN of the expected tokens, and return the
       /// number of tokens stored in YYARG.  If YYARG is null, return the
@@ -1955,19 +2129,19 @@ switch (yykind)
 
     /// Whether the given \c yypact_ value indicates a defaulted state.
     /// \param yyvalue   the value to check
-    static bool yy_pact_value_is_default_ (int yyvalue);
+    static bool yy_pact_value_is_default_ (int yyvalue) YY_NOEXCEPT;
 
     /// Whether the given \c yytable_ value indicates a syntax error.
     /// \param yyvalue   the value to check
-    static bool yy_table_value_is_error_ (int yyvalue);
+    static bool yy_table_value_is_error_ (int yyvalue) YY_NOEXCEPT;
 
-    static const signed char yypact_ninf_;
+    static const short yypact_ninf_;
     static const signed char yytable_ninf_;
 
     /// Convert a scanner token kind \a t to a symbol kind.
     /// In theory \a t should be a token_kind_type, but character literals
-    /// are valid, yet not members of the token_type enum.
-    static symbol_kind_type yytranslate_ (int t);
+    /// are valid, yet not members of the token_kind_type enum.
+    static symbol_kind_type yytranslate_ (int t) YY_NOEXCEPT;
 
     /// Convert the symbol name \a n to a form suitable for a diagnostic.
     static std::string yytnamerr_ (const char *yystr);
@@ -1990,7 +2164,7 @@ switch (yykind)
     static const short yypgoto_[];
 
     // YYDEFGOTO[NTERM-NUM].
-    static const short yydefgoto_[];
+    static const unsigned char yydefgoto_[];
 
     // YYTABLE[YYPACT[STATE-NUM]] -- What to do in state STATE-NUM.  If
     // positive, shift that token.  If negative, reduce the rule whose
@@ -1999,14 +2173,14 @@ switch (yykind)
 
     static const short yycheck_[];
 
-    // YYSTOS[STATE-NUM] -- The (internal number of the) accessing
-    // symbol of state STATE-NUM.
+    // YYSTOS[STATE-NUM] -- The symbol kind of the accessing symbol of
+    // state STATE-NUM.
     static const signed char yystos_[];
 
-    // YYR1[YYN] -- Symbol number of symbol that rule YYN derives.
+    // YYR1[RULE-NUM] -- Symbol kind of the left-hand side of rule RULE-NUM.
     static const signed char yyr1_[];
 
-    // YYR2[YYN] -- Number of symbols on the right hand side of rule YYN.
+    // YYR2[RULE-NUM] -- Number of symbols on the right-hand side of rule RULE-NUM.
     static const signed char yyr2_[];
 
 
@@ -2105,7 +2279,7 @@ switch (yykind)
       typedef typename S::size_type size_type;
       typedef typename std::ptrdiff_t index_type;
 
-      stack (size_type n = 200)
+      stack (size_type n = 200) YY_NOEXCEPT
         : seq_ (n)
       {}
 
@@ -2184,7 +2358,7 @@ switch (yykind)
       class slice
       {
       public:
-        slice (const stack& stack, index_type range)
+        slice (const stack& stack, index_type range) YY_NOEXCEPT
           : stack_ (stack)
           , range_ (range)
         {}
@@ -2234,14 +2408,14 @@ switch (yykind)
     void yypush_ (const char* m, state_type s, YY_MOVE_REF (symbol_type) sym);
 
     /// Pop \a n symbols from the stack.
-    void yypop_ (int n = 1);
+    void yypop_ (int n = 1) YY_NOEXCEPT;
 
     /// Constants.
     enum
     {
-      yylast_ = 217,     ///< Last index in yytable_.
-      yynnts_ = 17,  ///< Number of nonterminal symbols.
-      yyfinal_ = 33 ///< Termination state number.
+      yylast_ = 286,     ///< Last index in yytable_.
+      yynnts_ = 18,  ///< Number of nonterminal symbols.
+      yyfinal_ = 42 ///< Termination state number.
     };
 
 
@@ -2252,7 +2426,7 @@ switch (yykind)
 
   inline
   EvalParser::symbol_kind_type
-  EvalParser::yytranslate_ (int t)
+  EvalParser::yytranslate_ (int t) YY_NOEXCEPT
   {
     // YYTRANSLATE[TOKEN-NUM] -- Symbol number corresponding to
     // TOKEN-NUM as returned by yylex.
@@ -2290,15 +2464,16 @@ switch (yykind)
       15,    16,    17,    18,    19,    20,    21,    22,    23,    24,
       25,    26,    27,    28,    29,    30,    31,    32,    33,    34,
       35,    36,    37,    38,    39,    40,    41,    42,    43,    44,
-      45,    46,    47,    48,    49,    50,    51,    52,    53,    54
+      45,    46,    47,    48,    49,    50,    51,    52,    53,    54,
+      55,    56,    57,    58,    59,    60,    61,    62,    63
     };
     // Last valid token kind.
-    const int code_max = 309;
+    const int code_max = 318;
 
     if (t <= 0)
       return symbol_kind::S_YYEOF;
     else if (t <= code_max)
-      return YY_CAST (symbol_kind_type, translate_table[t]);
+      return static_cast <symbol_kind_type> (translate_table[t]);
     else
       return symbol_kind::S_YYUNDEF;
   }
@@ -2362,6 +2537,7 @@ switch (yykind)
 
 
 
+
   template <typename Base>
   EvalParser::symbol_kind_type
   EvalParser::basic_symbol<Base>::type_get () const YY_NOEXCEPT
@@ -2369,6 +2545,7 @@ switch (yykind)
     return this->kind ();
   }
 
+
   template <typename Base>
   bool
   EvalParser::basic_symbol<Base>::empty () const YY_NOEXCEPT
@@ -2434,13 +2611,13 @@ switch (yykind)
 
   // by_kind.
   inline
-  EvalParser::by_kind::by_kind ()
+  EvalParser::by_kind::by_kind () YY_NOEXCEPT
     : kind_ (symbol_kind::S_YYEMPTY)
   {}
 
 #if 201103L <= YY_CPLUSPLUS
   inline
-  EvalParser::by_kind::by_kind (by_kind&& that)
+  EvalParser::by_kind::by_kind (by_kind&& that) YY_NOEXCEPT
     : kind_ (that.kind_)
   {
     that.clear ();
@@ -2448,18 +2625,20 @@ switch (yykind)
 #endif
 
   inline
-  EvalParser::by_kind::by_kind (const by_kind& that)
+  EvalParser::by_kind::by_kind (const by_kind& that) YY_NOEXCEPT
     : kind_ (that.kind_)
   {}
 
   inline
-  EvalParser::by_kind::by_kind (token_kind_type t)
+  EvalParser::by_kind::by_kind (token_kind_type t) YY_NOEXCEPT
     : kind_ (yytranslate_ (t))
   {}
 
+
+
   inline
   void
-  EvalParser::by_kind::clear ()
+  EvalParser::by_kind::clear () YY_NOEXCEPT
   {
     kind_ = symbol_kind::S_YYEMPTY;
   }
@@ -2479,6 +2658,7 @@ switch (yykind)
     return kind_;
   }
 
+
   inline
   EvalParser::symbol_kind_type
   EvalParser::by_kind::type_get () const YY_NOEXCEPT
@@ -2486,9 +2666,10 @@ switch (yykind)
     return this->kind ();
   }
 
+
 #line 14 "parser.yy"
 } } // isc::eval
-#line 2492 "parser.h"
+#line 2673 "parser.h"
 
 
 
