@@ -1,4 +1,4 @@
-// Copyright (C) 2013-2018 Internet Systems Consortium, Inc. ("ISC")
+// Copyright (C) 2013-2023 Internet Systems Consortium, Inc. ("ISC")
 //
 // This Source Code Form is subject to the terms of the Mozilla Public
 // License, v. 2.0. If a copy of the MPL was not distributed with this
@@ -8,13 +8,13 @@
 #define D_CONTROLLER_H
 
 #include <asiolink/io_service.h>
+#include <asiolink/io_service_signal.h>
 #include <cc/data.h>
 #include <exceptions/exceptions.h>
 #include <log/logger_support.h>
 #include <process/daemon.h>
 #include <process/d_log.h>
 #include <process/d_process.h>
-#include <process/io_service_signal.h>
 
 #include <boost/shared_ptr.hpp>
 #include <boost/noncopyable.hpp>
@@ -73,7 +73,6 @@ public:
         isc::Exception(file, line, what) { };
 };
 
-
 /// @brief Defines a shared pointer to DControllerBase.
 class DControllerBase;
 typedef boost::shared_ptr<DControllerBase> DControllerBasePtr;
@@ -123,9 +122,10 @@ public:
     /// 1. parse command line arguments
     /// 2. instantiate and initialize the application process
     /// 3. load the configuration file
-    /// 4. initialize signal handling
-    /// 5. start and wait on the application process event loop
-    /// 6. exit to the caller
+    /// 4. record the start timestamp
+    /// 5. initialize signal handling
+    /// 6. start and wait on the application process event loop
+    /// 7. exit to the caller
     ///
     /// It is intended to be called from main() and be given the command line
     /// arguments.
@@ -148,7 +148,8 @@ public:
     /// process object.
     /// ProcessRunError - A fatal error occurred while in the application
     /// process event loop.
-    virtual void launch(int argc, char* argv[], const bool test_mode);
+    /// @return The value from @c Daemon::getExitValue().
+    virtual int launch(int argc, char* argv[], const bool test_mode);
 
     /// @brief Instance method invoked by the configuration event handler and
     /// which processes the actual configuration update.  Provides behavioral
@@ -186,9 +187,6 @@ public:
     ///
     /// @code
     ///  { "<module-name>": {<module-config>}
-    ///
-    ///   # Logging element is optional
-    ///   ,"Logging": {<logger config}
     ///  }
     ///
     ///  where:
@@ -204,12 +202,11 @@ public:
     /// file content using an alternate parser.  If it returns an empty pointer
     /// than the JSON parsing providing by Element::fromJSONFile() is called.
     ///
-    /// Once parsed, the method looks for the Element "Logging" and, if present
-    /// uses it to configure logging.
-    ///
-    /// It then extracts the set of configuration elements for the
-    /// module-name that matches the controller's app_name_ and passes that
-    /// set into @c updateConfig() (or @c checkConfig()).
+    /// Once parsed, the method extracts the set of configuration
+    /// elements for the module-name that matches the controller's app_name_,
+    /// looks for the loggers entry and, if present uses it to configure
+    /// logging. It then passes that set into @c updateConfig() (or
+    /// @c checkConfig()).
     ///
     /// The file may contain an arbitrary number of other modules.
     ///
@@ -268,6 +265,18 @@ public:
     configGetHandler(const std::string& command,
                      isc::data::ConstElementPtr args);
 
+    /// @brief handler for config-hash-get command
+    ///
+    /// This method handles the config-hash-get command, which retrieves
+    /// the current configuration and returns it in response.
+    ///
+    /// @param command (ignored)
+    /// @param args (ignored)
+    /// @return hash of current configuration wrapped in a response
+    isc::data::ConstElementPtr
+    configHashGetHandler(const std::string& command,
+                         isc::data::ConstElementPtr args);
+
     /// @brief handler for config-write command
     ///
     /// This handle processes write-config command, which writes the
@@ -297,6 +306,30 @@ public:
     configTestHandler(const std::string& command,
                       isc::data::ConstElementPtr args);
 
+    /// @brief handler for config-reload command
+    ///
+    /// This method handles the config-reload command, which reloads
+    /// the configuration file.
+    ///
+    /// @param command (ignored)
+    /// @param args (ignored)
+    /// @return status of the command
+    isc::data::ConstElementPtr
+    configReloadHandler(const std::string& command,
+                        isc::data::ConstElementPtr args);
+
+    /// @brief handler for config-set command
+    ///
+    /// This method handles the config-set command, which loads
+    /// configuration specified in args parameter.
+    ///
+    /// @param command (ignored)
+    /// @param args configuration to be checked.
+    /// @return status of the command
+    isc::data::ConstElementPtr
+    configSetHandler(const std::string& command,
+                     isc::data::ConstElementPtr args);
+
     /// @brief handler for 'shutdown' command
     ///
     /// This method handles shutdown command. It initiates the shutdown procedure
@@ -308,6 +341,31 @@ public:
     shutdownHandler(const std::string& command,
                     isc::data::ConstElementPtr args);
 
+    /// @brief handler for server-tag-get command
+    ///
+    /// This method handles the server-tag-get command, which retrieves
+    /// the current server tag and returns it in response.
+    ///
+    /// @param command (ignored)
+    /// @param args (ignored)
+    /// @return current configuration wrapped in a response
+    isc::data::ConstElementPtr
+    serverTagGetHandler(const std::string& command,
+                        isc::data::ConstElementPtr args);
+
+    /// @brief handler for status-get command
+    ///
+    /// This method handles the status-get command, which retrieves
+    /// the server process information i.e. the pid and returns it in
+    /// response.
+    ///
+    /// @param command (ignored)
+    /// @param args (ignored)
+    /// @return process information wrapped in a response
+    isc::data::ConstElementPtr
+    statusGetHandler(const std::string& command,
+                     isc::data::ConstElementPtr args);
+
 protected:
     /// @brief Virtual method that provides derivations the opportunity to
     /// support additional command line options.  It is invoked during command
@@ -360,9 +418,8 @@ protected:
 
     /// @brief Application-level signal processing method.
     ///
-    /// This method is the last step in processing a OS signal occurrence.  It
-    /// is invoked when an IOSignal's internal timer callback is executed by
-    /// IOService.  It currently supports the following signals as follows:
+    /// This method is the last step in processing a OS signal occurrence.
+    /// It currently supports the following signals as follows:
     /// -# SIGHUP - instigates reloading the configuration file
     /// -# SIGINT - instigates a graceful shutdown
     /// -# SIGTERM - instigates a graceful shutdown
@@ -445,7 +502,6 @@ protected:
     /// @throw VersionMessage if the -v, -V or -W arguments is given.
     void parseArgs(int argc, char* argv[]);
 
-
     ///@brief Parse a given file into Elements
     ///
     /// This method provides a means for deriving classes to use alternate
@@ -528,37 +584,10 @@ protected:
     /// @brief Initializes signal handling
     ///
     /// This method configures the controller to catch and handle signals.
-    /// It instantiates an IOSignalQueue, registers @c osSignalHandler() as
-    /// the SignalSet "on-receipt" handler, and lastly instantiates a SignalSet
-    /// which listens for SIGHUP, SIGINT, and SIGTERM.
+    /// It instantiates a IOSignalSet which listens for SIGHUP, SIGINT, and
+    /// SIGTERM.
     void initSignalHandling();
 
-    /// @brief Handler for processing OS-level signals
-    ///
-    /// This method is installed as the SignalSet "on-receipt" handler. Upon
-    /// invocation, it uses the controller's IOSignalQueue to schedule an
-    /// IOSignal with for the given signal value.
-    ///
-    /// @param signum OS signal value (e.g. SIGINT, SIGUSR1 ...) to received
-    ///
-    /// @return SignalSet "on-receipt" handlers are required to return a
-    /// boolean indicating if the OS signal has been processed (true) or if it
-    /// should be saved for deferred processing (false).  Currently this
-    /// method processes all received signals, so it always returns true.
-    bool osSignalHandler(int signum);
-
-    /// @brief Handler for processing IOSignals
-    ///
-    /// This method is supplied as the callback when IOSignals are scheduled.
-    /// It fetches the IOSignal for the given sequence_id and then invokes
-    /// the virtual method, @c processSignal() passing it the signal value
-    /// obtained from the IOSignal.  This allows derivations to supply a
-    /// custom signal processing method, while ensuring IOSignalQueue
-    /// integrity.
-    ///
-    /// @param sequence_id id of the IOSignal instance "received"
-    void ioSignalHandler(IOSignalId sequence_id);
-
     /// @brief Fetches the current process
     ///
     /// @return a pointer to the current process instance.
@@ -581,6 +610,15 @@ protected:
     /// @return a string containing additional version info
     virtual std::string getVersionAddendum() { return (""); }
 
+    /// @brief Deals with other (i.e. not application name) global objects.
+    ///
+    /// Code shared between configuration handlers:
+    ///  - check obsolete or unknown (aka unsupported) objects.
+    ///
+    /// @param args Command arguments.
+    /// @return Error message or empty string.
+    std::string handleOtherObjects(isc::data::ConstElementPtr args);
+
 private:
     /// @brief Name of the service under control.
     /// This name is used as the configuration module name and appears in log
@@ -606,10 +644,10 @@ private:
     DProcessBasePtr process_;
 
     /// @brief Shared pointer to an IOService object, used for ASIO operations.
-    asiolink::IOServicePtr io_service_;
+    isc::asiolink::IOServicePtr io_service_;
 
-    /// @brief Queue for propagating caught signals to the IOService.
-    IOSignalQueuePtr io_signal_queue_;
+    /// @brief ASIO signal set.
+    isc::asiolink::IOSignalSetPtr io_signal_set_;
 
     /// @brief Singleton instance value.
     static DControllerBasePtr controller_;
@@ -619,7 +657,7 @@ private:
 friend class DControllerTest;
 };
 
-}; // namespace isc::process
-}; // namespace isc
+}  // namespace process
+}  // namespace isc
 
 #endif
