@@ -1,6 +1,6 @@
 #!/bin/sh
 
-# Copyright (C) 2019-2022 Internet Systems Consortium, Inc. ("ISC")
+# Copyright (C) 2019-2023 Internet Systems Consortium, Inc. ("ISC")
 #
 # This Source Code Form is subject to the terms of the Mozilla Public
 # License, v. 2.0. If a copy of the MPL was not distributed with this
@@ -25,10 +25,10 @@ prefix="/usr"
 
 # Include utilities. Use installed version if available and
 # use build version if it isn't.
-if [ -e ${prefix}/share/kea/scripts/admin-utils.sh ]; then
+if test -f "${prefix}/share/kea/scripts/admin-utils.sh"; then
     . "${prefix}/share/kea/scripts/admin-utils.sh"
 else
-    . "/build/kea/src/kea-2.2.0/src/bin/admin/admin-utils.sh"
+    . "/build/kea/src/kea-2.4.1/src/bin/admin/admin-utils.sh"
 fi
 
 # First argument must be the expected schema version <major>.<minor>
@@ -69,8 +69,10 @@ DELETE FROM ipv6_reservations;
 DELETE FROM dhcp6_options;
 DELETE FROM lease4;
 DELETE FROM lease4_stat;
+DELETE FROM lease4_pool_stat;
 DELETE FROM lease6;
 DELETE FROM lease6_stat;
+DELETE FROM lease6_pool_stat;
 DELETE FROM logs;
 DELETE FROM lease4_stat_by_client_class;
 DELETE FROM lease6_stat_by_client_class;
@@ -113,5 +115,8 @@ DELETE FROM dhcp6_client_class_server;
 DELETE FROM dhcp6_client_class;
 DELETE FROM dhcp6_server WHERE tag != 'all'; -- preserve the special tag
 
+DELETE FROM lease6_relay_id;
+DELETE FROM lease6_remote_id;
+
 COMMIT;
 EOF
